from datetime import datetime, time
from typing import Literal, Optional
from pydantic import BaseModel, Field


class Status(BaseModel):
    satzart: Literal["STATUS"] = "STATUS"
    referenz: str
    tladenr: str
    aufnr: int
    aufposnr: Optional[int] = None
    refpos: Optional[str] = None
    aufintnr: Optional[int] = None
    statusnr: str
    diffmenge: Optional[float] = None
    diffeinh: Optional[str] = None
    hinweis: Optional[str] = None
    statusdat: Optional[datetime] = None
    statuszeit: Optional[time] = None
    ean: Optional[str] = None
    refnr: Optional[str] = None
    ffnr: Optional[int] = None
    lkw: Optional[str] = None
    belvondat: Optional[datetime] = None
    tournr: Optional[str] = None
    tatsgew: Optional[float] = None
    waehrung: Optional[str] = None
    gsbetrag: Optional[float] = None
    rebetrag: Optional[float] = None
    aufart: Optional[str] = None
    belvonzeit: Optional[time] = None
    statusbez: Optional[str] = None
    kstelle: Optional[int] = None
    ktotab: Optional[str] = None
    wtourid: Optional[int] = None
    wauftragid: Optional[int] = None
    wauftposid: Optional[int] = None
    ktraeger: Optional[int] = None
    entbisdat: Optional[datetime] = None
    entbiszeit: Optional[time] = None
    kommnr: Optional[str] = None
    aenddat: Optional[datetime] = None
    aendus: Optional[str] = None
    statusart: Optional[int] = None
    quitgeber: Optional[str] = None
    aufunternr: Optional[int] = None
    liefnr: Optional[str] = None
    ffpausch: Optional[float] = None
    stalfdnr: Optional[int] = None
    lkwpolkz: Optional[str] = None
    anh: Optional[str] = None
    anhpolkz: Optional[str] = None
    fahld: Optional[int] = None
    bfahld: Optional[int] = None



import logging
import os
import shutil
import tempfile
from typing import Tuple, Union
from pydantic.dataclasses import dataclass

from tms_integration.utils.sftp import SftpBase

from .models import WindispoCreateJob, WindispoExportCreateJob

logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s [%(levelname)s] %(message)s"
)


@dataclass
class Windispo(SftpBase):
    import_dest_folder: str
    output_target_folder: str = ""

    def import_auftrag(self, payload: WindispoCreateJob, import_prefix: str | None = None):
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="utf-8",
            prefix=import_prefix,
            suffix=".xml",
            delete=False,
        ) as tmp_file:
            tmp_file.write(payload.generate_xml())
            tmp_file.close()
            self.import_file(tmp_file.name, self.import_dest_folder)

    @staticmethod
    def _normalize_identifier(value: str) -> str:
        return os.path.basename(value).strip().lower()

    def export_auftrag(
        self, identifier: str
    ) -> Union[Tuple[None, None], Tuple[str, WindispoExportCreateJob]]:
        if not self.output_target_folder:
            return None, None

        normalized_identifier = self._normalize_identifier(identifier)
        output_files = self.get_all_files(self.output_target_folder)
        dest_path = os.path.join(os.getcwd(), "tmp", "output")
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        os.makedirs(dest_path)

        for remote_file in output_files:
            filename = os.path.basename(remote_file)
            local_path = os.path.join(dest_path, filename)
            try:
                self.export_file(remote_file, local_path)
                with open(local_path, "r", encoding="utf-8") as xml_file:
                    payload = WindispoExportCreateJob.from_xml(xml_file.read())

                if (
                    self._normalize_identifier(payload.header.processed_file)
                    == normalized_identifier
                ):
                    return remote_file, payload
            except Exception:
                logging.exception(f"File [{remote_file}] cannot be accessed")

        return None, None



from pydantic import BaseModel, Field
from typing import Optional


class CustomModel(BaseModel):
    """
    Base class for custom models that can be subclassed to define
    custom record types for the TMS integration system.
    
    Subclasses should override the satzart field to define their
    own record type identifier.
    """
    satzart: str = Field(..., description="Record type identifier")
    referenz: str = Field(..., description="Reference identifier")
    
    class Config:
        # Allow arbitrary types for extensibility
        arbitrary_types_allowed = True
        # Allow extra fields in subclasses
        extra = 'allow'



import unittest
import xml.etree.ElementTree as ET

from tms_integration.windispo.models import (
    WindispoCreateJob,
    WindispoExportCreateJob,
    WindispoFile,
    WindispoHeader,
    WindispoJob,
    WindispoJobDetail,
)


class TestWindispoImportXml(unittest.TestCase):
    def test_generate_valid_createjob_v1_xml(self):
        payload = WindispoCreateJob(
            header=WindispoHeader(
                KdNr="39",
                MsgTime="20241127151739",
                MsgFrom="HB",
                ProcessType="new",
            ),
            job=WindispoJob(
                JobOwner="rs@windispo.at",
                ExtID="aaa1",
                AfKdNr="3",
                AfName1="Neele-Vat Transport B.V.",
                AfStraße="Bierbrouwerstraat 2",
                AfLand="NL",
                AfPlz="3194",
                AfOrt="Ap Hoogvliet",
                FzKdNr="3",
                FzName1="Neele-Vat Transport B.V.",
                FzStraße="Bierbrouwerstraat 2",
                FzLand="NL",
                FzPlz="3194",
                FzOrt="Ap Hoogvliet",
                Referenz="1802176771 FTL 90",
                FzgArt1="Planensattel",
                Whrg="EUR",
                jobdetail=[
                    WindispoJobDetail(
                        ZNr=1,
                        Laden=1,
                        Name1="Neele-Vat Transport BV",
                        Straße="Seattleweg 3",
                        Land="NL",
                        Plz="3195",
                        Ort="Pernis",
                        TerminVon="202208171100",
                        TerminBis="202208171430",
                        FixTermin=0,
                        Ladungsanzahl="66.00",
                        Ladungsart="Euro-Pal.",
                        Ladungsbezeichnung="Foodstuff",
                        Gewicht="4854.00",
                        ldm="13.60",
                        cbm="0.00",
                        Referenz="400154657",
                    ),
                    WindispoJobDetail(
                        ZNr=2,
                        Laden=2,
                        Name1="LSI Germany GmbH C/O Dachser",
                        Straße="Thomas Dachserstraße 4",
                        Land="DE",
                        Plz="90475",
                        Ort="Nürnberg",
                        TerminVon="202208180800",
                        TerminBis="202208181200",
                        FixTermin=0,
                        Ladungsanzahl="66.00",
                        Ladungsart="Euro-Pal.",
                        Ladungsbezeichnung="Foodstuff",
                        Gewicht="4854.00",
                        ldm="13.60",
                        cbm="0.00",
                        Referenz="PEU09722",
                    ),
                ],
                file=[
                    WindispoFile(
                        idx=1,
                        DocumentExtension=".msg",
                        AblageDatum="202208160922",
                        Betreff="Transport opdracht",
                        content="BASE64_CONTENT",
                    )
                ],
            ),
            jobcount=1,
        )

        xml_output = payload.generate_xml()
        root = ET.fromstring(xml_output)

        self.assertEqual(root.tag, "createjob.v1")
        self.assertEqual(root.findtext("jobcount"), "1")

        header = root.find("header")
        self.assertIsNotNone(header)
        assert header is not None  # for type checker
        self.assertEqual(header.findtext("KdNr"), "39")
        self.assertEqual(header.findtext("ProcessType"), "new")

        job = root.find("job")
        self.assertIsNotNone(job)
        assert job is not None  # for type checker
        self.assertEqual(job.findtext("AfStraße"), "Bierbrouwerstraat 2")
        self.assertEqual(job.findtext("FzStraße"), "Bierbrouwerstraat 2")

        jobdetails = job.findall("jobdetail")
        self.assertEqual(len(jobdetails), 2)
        self.assertEqual(job.findtext("jobdetailcount"), "2")

        files = job.findall("file")
        self.assertEqual(len(files), 1)
        self.assertEqual(job.findtext("filecount"), "1")
        self.assertEqual(files[0].attrib.get("idx"), "1")
        self.assertEqual(files[0].attrib.get("DocumentExtension"), ".msg")
        self.assertEqual(files[0].text, "BASE64_CONTENT")


class TestWindispoExportXml(unittest.TestCase):
    def test_parse_success_response(self):
        xml_response = (
            '<?xml version="1.0" encoding="utf-8"?>'
            "<createjob.v1>"
            "<header>"
            "<KdNr>71</KdNr>"
            "<MsgTime>20250219152455</MsgTime>"
            "<MsgFrom>TRAW</MsgFrom>"
            "<ProcessType>new accepted</ProcessType>"
            "<ProcessedFile>647496.xml</ProcessedFile>"
            "</header>"
            "<job>"
            "<DocID>2668/00</DocID>"
            "<ExtID>aaa1</ExtID>"
            "<AfKdNr>1487</AfKdNr>"
            "</job>"
            "<jobcount>1</jobcount>"
            "</createjob.v1>"
        )

        payload = WindispoExportCreateJob.from_xml(xml_response)

        self.assertEqual(payload.header.kd_nr, "71")
        self.assertEqual(payload.header.processed_file, "647496.xml")
        self.assertEqual(payload.job.doc_id, "2668/00")
        self.assertEqual(payload.job.ext_id, "aaa1")
        self.assertEqual(payload.job.af_kd_nr, "1487")
        self.assertIsNone(payload.job.error)

    def test_parse_failure_response(self):
        xml_response = (
            '<?xml version="1.0" encoding="utf-8"?>'
            "<createjob.v1>"
            "<header>"
            "<KdNr>71</KdNr>"
            "<MsgTime>20250219153331</MsgTime>"
            "<MsgFrom>TRAW</MsgFrom>"
            "<ProcessType>new failed</ProcessType>"
            "<ProcessedFile>647497.xml</ProcessedFile>"
            "</header>"
            "<job>"
            "<ExtID>aaa2</ExtID>"
            "<AfKdNr>3</AfKdNr>"
            "<Error><ErrorMsg>ExtID already used</ErrorMsg></Error>"
            "</job>"
            "<jobcount>1</jobcount>"
            "</createjob.v1>"
        )

        payload = WindispoExportCreateJob.from_xml(xml_response)

        self.assertEqual(payload.header.process_type, "new failed")
        self.assertIsNone(payload.job.doc_id)
        self.assertEqual(payload.job.ext_id, "aaa2")
        self.assertEqual(payload.job.af_kd_nr, "3")
        self.assertIsNotNone(payload.job.error)
        self.assertEqual(payload.job.error.error_msg, "ExtID already used")


if __name__ == "__main__":
    unittest.main()



from .createjob import (
    WindispoCreateJob,
    WindispoFile,
    WindispoHeader,
    WindispoJob,
    WindispoJobDetail,
)
from .exportjob import (
    WindispoExportCreateJob,
    WindispoExportError,
    WindispoExportHeader,
    WindispoExportJob,
)



from pydantic import BaseModel, field_validator
from typing import Optional, Literal
from datetime import datetime


class Start(BaseModel):
    satzart: Literal["START"] = "START"
    referenz: str
    erstellt: datetime
    eigiln: Optional[str] = None
    eigkreid: Optional[str] = None
    eigname1: str
    expiln: Optional[str] = None
    expempid: Optional[str] = None
    expname1: str
    impart: Optional[int] = None
    maplfdnr: Optional[int] = None
    status: Optional[str] = None
    versinfo: Optional[str] = None
    skalierungtext1: Optional[str] = None
    skalierungtext2: Optional[str] = None
    skalierungtext3: Optional[str] = None
    skalierungzahl1: Optional[int] = None
    skalierungzahl2: Optional[int] = None
    skalierungdatum1: Optional[datetime] = None
    skalierungdatum2: Optional[datetime] = None

    @field_validator("erstellt", "skalierungdatum1", "skalierungdatum2", mode="before")
    @classmethod
    def parse_datetime(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%Y%m%d%H%M%S")
        return value

    @field_validator("status")
    @classmethod
    def validate_status(cls, value):
        if value and value not in {"D", "L", "Z", "E", "F"}:
            raise ValueError("Invalid status value")
        return value



from typing import Any, List, Optional
from datetime import date, datetime

from pydantic import BaseModel, field_validator
import xml.etree.ElementTree as ET
from xml.dom.minidom import parseString

from tms_integration.utils.xml import XmlAttribute, to_xml_element


class BaseCarloClass(BaseModel):
    class Config:
        arbitrary_types_allowed = True

    @field_validator("Action", mode="before")
    @classmethod
    def set_action(cls, v: Any) -> Any:
        return XmlAttribute(v)


class Address(BaseCarloClass):
    Street: str
    HouseNumber: str
    Location1: str
    ZipCode: str
    IsoTwoCharacterCountryCode: str


class OriginalBusinessPartner(BaseCarloClass):
    Action: XmlAttribute
    GlobalLocationNumber: str
    Name1: str
    Addresses: Address


class Receiver(BaseCarloClass):
    OriginalBusinessPartner: OriginalBusinessPartner


class Sender(BaseCarloClass):
    OriginalBusinessPartner: OriginalBusinessPartner


class Package(BaseCarloClass):
    Matchcode: Optional[str] = None


class SsccCurrent(BaseCarloClass):
    Code: str


class ConsignmentItem(BaseCarloClass):
    PositionNumber: int
    Quantity: int
    Package: Package
    EffectiveWeightInKilogram: float
    StoragePlaces: Optional[float] = None
    Meter: Optional[float] = None
    SsccCurrents: Optional[List[SsccCurrent]] = None


class Times(BaseCarloClass):
    LieferdatumStart: Optional[datetime] = None
    LieferdatumEnd: Optional[datetime] = None
    LadedatumStart: Optional[datetime] = None
    LadedatumEnd: Optional[datetime] = None


class InformationClass(BaseCarloClass):
    Info10: str


class EditStatusClass(BaseCarloClass):
    Matchcode: str = ""


class ConsignmentCustomFields(BaseCarloClass):
    Action: XmlAttribute = "updateorcreate"
    CustomTypeValue1: str = ""
    ConsignmentReference1: str = ""


class Consignment(BaseCarloClass):
    Number: int
    ConsignmentReference2: str
    Times: Times
    Incoterms: int
    Receiver: Receiver
    Sender: Sender
    ConsignmentItems: Optional[List[ConsignmentItem]] = None
    ConsignmentReference1: Optional[str] = None
    ExternalNumber: Optional[str] = None
    ContainsDangerousGoods: Optional[bool] = None
    Information: Optional[InformationClass] = None
    EditStatus: Optional[EditStatusClass] = None
    AdditionalFields: Optional[ConsignmentCustomFields] = None
    Matchcode: Optional[str] = None


class Customer(BaseCarloClass):
    GlobalLocationNumber: str


class NormalOrder(BaseCarloClass):
    Action: XmlAttribute = "updateorcreate"
    ExternalNumber: str
    Date: date
    Info1: str = ""
    Customer: Customer
    Consignments: List[Consignment]


def remove_interim_tag(element: ET.Element, tag: str) -> None:
    """Recursively remove an interim tag from the element and replace it with its children, which get the interim element's name."""
    for child in list(
        element
    ):  # Convert to list to safely modify the tree during iteration
        if child.tag == tag:
            element.remove(child)  # Remove the interim element
            for grandchild in list(child):  # Again, convert to list to safely iterate
                grandchild.tag = (
                    tag  # Rename the grandchild tag to the interim element's tag name
                )
                element.append(
                    grandchild
                )  # Append the grandchild to the parent element
        else:
            remove_interim_tag(child, tag)  # Recurse into child elements


class NormalOrderData(BaseCarloClass):
    NormalOrder: NormalOrder

    def generate_xml(self) -> str:
        # Create the root element
        root = to_xml_element("NormalOrderData", self)

        # Remove and replace tags according to Carlo schema
        remove_interim_tag(root, "ConsignmentItems")
        remove_interim_tag(root, "Consignments")

        # Generate the XML string
        xml_str = ET.tostring(root, encoding="utf-8").decode("utf-8")

        # Add XML declaration and namespaces manually
        xml_declaration = '<?xml version="1.0" encoding="utf-8"?>'

        final_xml = f"{xml_declaration}{xml_str}"

        # Pretty-print the XML and return it
        dom = parseString(final_xml)
        return dom.toprettyxml(indent="  ")



import xml.etree.ElementTree as ET
from typing import List, Optional
from xml.dom.minidom import parseString

from pydantic import BaseModel, Field


class WindispoBaseModel(BaseModel):
    model_config = {"populate_by_name": True}


def _append_text_element(parent: ET.Element, tag: str, value: Optional[object]) -> None:
    element = ET.SubElement(parent, tag)
    if value is not None:
        element.text = str(value)


class WindispoHeader(WindispoBaseModel):
    kd_nr: str = Field(alias="KdNr")
    msg_time: str = Field(alias="MsgTime")
    msg_from: str = Field(alias="MsgFrom")
    process_type: str = Field(default="new", alias="ProcessType")


class WindispoJobDetail(WindispoBaseModel):
    z_nr: int = Field(alias="ZNr")
    laden: int = Field(alias="Laden")
    anrede: Optional[str] = Field(default=None, alias="Anrede")
    name1: str = Field(alias="Name1")
    name2: Optional[str] = Field(default=None, alias="Name2")
    name3: Optional[str] = Field(default=None, alias="Name3")
    strasse: str = Field(alias="Straße")
    land: str = Field(alias="Land")
    plz: str = Field(alias="Plz")
    ort: str = Field(alias="Ort")
    termin_von: Optional[str] = Field(default=None, alias="TerminVon")
    termin_bis: Optional[str] = Field(default=None, alias="TerminBis")
    fix_termin: Optional[int] = Field(default=None, alias="FixTermin")
    ladungsanzahl: Optional[str] = Field(default=None, alias="Ladungsanzahl")
    ladungsart: Optional[str] = Field(default=None, alias="Ladungsart")
    ladungsbezeichnung: Optional[str] = Field(default=None, alias="Ladungsbezeichnung")
    gewicht: Optional[str] = Field(default=None, alias="Gewicht")
    ldm: Optional[str] = Field(default=None, alias="ldm")
    cbm: Optional[str] = Field(default=None, alias="cbm")
    referenz: Optional[str] = Field(default=None, alias="Referenz")
    info1: Optional[str] = Field(default=None, alias="Info1")
    container_nr1: Optional[str] = Field(default=None, alias="ContainerNr1")
    container_nr2: Optional[str] = Field(default=None, alias="ContainerNr2")
    geo_lat: Optional[str] = Field(default=None, alias="GeoLat")
    geo_lng: Optional[str] = Field(default=None, alias="GeoLng")


class WindispoFile(WindispoBaseModel):
    idx: int
    document_extension: Optional[str] = Field(default=None, alias="DocumentExtension")
    ablage_datum: Optional[str] = Field(default=None, alias="AblageDatum")
    betreff: Optional[str] = Field(default=None, alias="Betreff")
    content: str = ""

    def to_xml_element(self) -> ET.Element:
        element = ET.Element("file")
        element.set("idx", str(self.idx))
        if self.document_extension is not None:
            element.set("DocumentExtension", self.document_extension)
        if self.ablage_datum is not None:
            element.set("AblageDatum", self.ablage_datum)
        if self.betreff is not None:
            element.set("Betreff", self.betreff)
        element.text = self.content
        return element


class WindispoJob(WindispoBaseModel):
    job_owner: str = Field(alias="JobOwner")
    ext_id: str = Field(alias="ExtID")

    af_kd_nr: str = Field(alias="AfKdNr")
    af_anrede: Optional[str] = Field(default=None, alias="AfAnrede")
    af_name1: str = Field(alias="AfName1")
    af_name2: Optional[str] = Field(default=None, alias="AfName2")
    af_name3: Optional[str] = Field(default=None, alias="AfName3")
    af_strasse: str = Field(alias="AfStraße")
    af_land: str = Field(alias="AfLand")
    af_plz: str = Field(alias="AfPlz")
    af_ort: str = Field(alias="AfOrt")

    fz_kd_nr: str = Field(alias="FzKdNr")
    fz_anrede: Optional[str] = Field(default=None, alias="FzAnrede")
    fz_name1: str = Field(alias="FzName1")
    fz_name2: Optional[str] = Field(default=None, alias="FzName2")
    fz_name3: Optional[str] = Field(default=None, alias="FzName3")
    fz_strasse: str = Field(alias="FzStraße")
    fz_land: str = Field(alias="FzLand")
    fz_plz: str = Field(alias="FzPlz")
    fz_ort: str = Field(alias="FzOrt")

    referenz: str = Field(alias="Referenz")
    info1: Optional[str] = Field(default=None, alias="Info1")
    info2: Optional[str] = Field(default=None, alias="Info2")
    tscaa_id: Optional[str] = Field(default=None, alias="TScaaID")
    fzg_art1: Optional[str] = Field(default=None, alias="FzgArt1")
    fzg_art2: Optional[str] = Field(default=None, alias="FzgArt2")
    kuehltemperatur: Optional[str] = Field(default=None, alias="Kuehltemperatur")
    job_sg: str = Field(default="0", alias="jobSG")
    whrg: str = Field(default="EUR", alias="Whrg")

    jobdetails: List[WindispoJobDetail] = Field(default_factory=list, alias="jobdetail")
    files: List[WindispoFile] = Field(default_factory=list, alias="file")

    jobdetailcount: Optional[int] = Field(default=None, alias="jobdetailcount")
    filecount: Optional[int] = Field(default=None, alias="filecount")


class WindispoCreateJob(WindispoBaseModel):
    header: WindispoHeader
    job: WindispoJob
    jobcount: int = 1

    def _serialize_model_fields(
        self,
        parent: ET.Element,
        model: WindispoBaseModel,
        skip: Optional[set] = None,
    ) -> None:
        excluded = skip or set()
        for field_name, field_info in model.__class__.model_fields.items():
            if field_name in excluded:
                continue
            tag = field_info.alias or field_name
            value = getattr(model, field_name)
            _append_text_element(parent, tag, value)

    def to_xml_element(self) -> ET.Element:
        root = ET.Element("createjob.v1")

        header_element = ET.SubElement(root, "header")
        self._serialize_model_fields(header_element, self.header)

        job_element = ET.SubElement(root, "job")
        self._serialize_model_fields(
            job_element,
            self.job,
            skip={"jobdetails", "files", "jobdetailcount", "filecount"},
        )

        for detail in self.job.jobdetails:
            detail_element = ET.SubElement(job_element, "jobdetail")
            self._serialize_model_fields(detail_element, detail)

        for file in self.job.files:
            job_element.append(file.to_xml_element())

        _append_text_element(job_element, "jobdetailcount", len(self.job.jobdetails))
        _append_text_element(job_element, "filecount", len(self.job.files))

        _append_text_element(root, "jobcount", self.jobcount)
        return root

    def generate_xml(self) -> str:
        root = self.to_xml_element()
        xml_bytes = ET.tostring(root, encoding="utf-8")
        pretty_xml = parseString(xml_bytes).toprettyxml(indent="  ", encoding="utf-8")
        return pretty_xml.decode("utf-8")



import tempfile
from pydantic.dataclasses import dataclass
from tms_integration.utils.sftp import SftpBase

from .models.order import NormalOrderData


@dataclass
class SoloplanCarlo(SftpBase):
    import_dest_folder: str

    def import_auftrag(self, payload: NormalOrderData, import_prefix: str = None):
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="utf-8",
            prefix=import_prefix,
            suffix=".xml",
            delete=False,
        ) as tmp_file:
            tmp_file.write(payload.generate_xml())
            tmp_file.close()
            self.import_file(tmp_file.name, self.import_dest_folder)

    def export_auftrag(self):
        raise NotImplementedError



from typing import Union
from .adr import Adr
from .auftrag import Auftrag
from .ende import Ende
from .ggut import Ggut
from .ladeli import Ladeli
from .lademi import Lademi
from .posit import Posit
from .start import Start
from .text import Text
from .geb import Geb
from .dmsdok import DmsDok
from .dmsref import DmsRef
from .dmssw import DmsSw
from .status import Status
from .custom import CustomModel


RecordTypes = Union[
    Auftrag,
    Ende,
    Ggut,
    Ladeli,
    Lademi,
    Posit,
    Start,
    Text,
    Adr,
    Geb,
    DmsDok,
    DmsRef,
    DmsSw,
    Status,
    CustomModel,
]



from .windispo import Windispo
from .models import (
	WindispoCreateJob,
	WindispoExportCreateJob,
	WindispoExportError,
	WindispoExportHeader,
	WindispoExportJob,
	WindispoFile,
	WindispoHeader,
	WindispoJob,
	WindispoJobDetail,
)



from pydantic import BaseModel, field_validator
from typing import Optional, Literal
from datetime import datetime


class Text(BaseModel):
    satzart: Literal["TEXT"] = "TEXT"
    referenz: str
    tladenr: str
    aufnr: int
    txtartnr: int
    txtstr1: str
    gtext: Optional[str] = None  # changed from Optional[bool]
    txtstr2: Optional[str] = None
    txtstr3: Optional[str] = None
    txtstr4: Optional[str] = None
    txtstr5: Optional[str] = None
    txtdatum: Optional[datetime] = None

    @field_validator("txtdatum", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%Y%m%d")
        return value

    @field_validator("gtext")
    @classmethod
    def validate_gtext(cls, value):
        if value in {True, "J", "j"}:
            return "J"
        elif value in {False, "N", "n"}:
            return "N"
        elif value is None:
            return None
        raise ValueError("gtext must be 'J', 'N', True, False, or None")



from pydantic import BaseModel, field_validator
from typing import Literal, Optional


class Ggut(BaseModel):
    satzart: Literal["GGUT"] = "GGUT"
    tladenr: str
    aufnr: int
    lfdnrggut: int
    aufposnr: Optional[int] = None
    klasse: str
    klassecode: Optional[str] = None
    verpackgrp: Optional[str] = None
    befkat: Optional[str] = None
    befvorschr: Optional[str] = None
    unnummer: str
    nrgefahr: Optional[str] = None
    begrmenge: Optional[str] = None
    verpart: Optional[str] = None
    anzvpe: Optional[float] = None
    gewicht: Optional[float] = None
    gewichtbrt: Optional[float] = None
    gewichtnet: Optional[float] = None
    cdmanz: Optional[float] = None
    flammpunkt: Optional[float] = None
    multiplika: Optional[float] = None
    stoffname: Optional[str] = None
    chemname: Optional[str] = None
    buchstabe: Optional[str] = None
    nebengf1: Optional[str] = None
    nebengf2: Optional[str] = None
    nebengf3: Optional[str] = None
    nve: Optional[str] = None
    sonvorschr: Optional[str] = None
    nagtext: Optional[str] = None
    gefahrzett: Optional[str] = None
    tunnelcode: Optional[str] = None
    punkte: Optional[float] = None
    herausgeb: Optional[str] = None
    liskatnr: Optional[int] = None
    unnr_lfdnr: Optional[int] = None
    un_lfdnr: Optional[int] = None
    freimenge: Optional[str] = None
    vertraeggr: Optional[str] = None
    umweltgef: Optional[bool] = None

    @field_validator("umweltgef")
    @classmethod
    def validate_boolean(cls, value):
        if value not in {None, True, False}:
            raise ValueError("Invalid boolean value")
        if value == False:
            return "N"  # Nein
        elif value == True:
            return "J"  # Ja
        return value



import os
import shutil
import tempfile
import logging
from typing import Union, Tuple, List
from pydantic.dataclasses import dataclass
from tms_integration.utils.sftp import SftpBase

from .models import LisIn

logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s [%(levelname)s] %(message)s"
)


@dataclass
class LisWinSped(SftpBase):
    import_dest_folder: str
    output_target_folder: str

    def import_auftrag(self, payload: LisIn, import_prefix: str = None):
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="cp1252",  # FORCE ANSI encoding
            prefix=import_prefix,
            suffix=".txt",
            delete=False,
        ) as tmp_file:
            tmp_file.write(payload.generate_txt())
            tmp_file.close()
            self.import_file(tmp_file.name, self.import_dest_folder)

    def import_document(
        self, dms_payload, file: str, import_prefix: str | None = None
    ) -> Tuple[str, str]:
        """Returns the generated import file name and text."""
        import_file_name: str = ""
        import_file_text: str = ""
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="cp1252",  # FORCE ANSI encoding
            prefix=import_prefix,
            suffix=".txt",
            delete=False,
        ) as tmp_file:
            import_file_name = tmp_file.name
            import_file_text = dms_payload.generate_txt()
            tmp_file.write(import_file_text)
            tmp_file.close()
            self.import_file(import_file_name, self.import_dest_folder)
        
        # send the file as well
        self.import_file(file, self.import_dest_folder)

        return import_file_name, import_file_text

    def export_auftrag(
        self, identifier: str
    ) -> Union[Tuple[None, None], Tuple[str, str]]:
        output_files = self.get_all_files(self.output_target_folder)
        dest_path = os.path.join(os.getcwd(), "tmp", "output")
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        os.makedirs(dest_path)
        for file in output_files:
            filename = os.path.basename(file)
            try:
                self.export_file(file, os.path.join(dest_path, filename))
                with open(
                    os.path.join(dest_path, filename), "r", encoding="cp1252"
                ) as txt:
                    text = txt.read()
                    if identifier in text:
                        return (file, text)
            except Exception:
                logging.exception(f"File [{file}] cannot be accessed")

        return None, None

    def export_auftrag_tour(self, identifier: str) -> List[Tuple[str, str]]:
        output: List[Tuple[str, str]] = []
        output_files = self.get_all_files(self.output_target_folder)
        dest_path = os.path.join(os.getcwd(), "tmp", "output")
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        os.makedirs(dest_path)
        for file in output_files:
            filename = os.path.basename(file)
            try:
                self.export_file(file, os.path.join(dest_path, filename))
                with open(
                    os.path.join(dest_path, filename), "r", encoding="cp1252"
                ) as txt:
                    text = txt.read()
                    if identifier in text:
                        output.append((file, text))
            except Exception:
                logging.exception(f"File [{file}] cannot be accessed")

        return output



import xml.etree.ElementTree as ET
from typing import Optional

from pydantic import BaseModel, Field


class WindispoExportBaseModel(BaseModel):
    model_config = {"populate_by_name": True}


class WindispoExportError(WindispoExportBaseModel):
    error_msg: str = Field(alias="ErrorMsg")


class WindispoExportHeader(WindispoExportBaseModel):
    kd_nr: str = Field(alias="KdNr")
    msg_time: str = Field(alias="MsgTime")
    msg_from: str = Field(alias="MsgFrom")
    process_type: str = Field(alias="ProcessType")
    processed_file: str = Field(alias="ProcessedFile")


class WindispoExportJob(WindispoExportBaseModel):
    doc_id: Optional[str] = Field(default=None, alias="DocID")
    ext_id: Optional[str] = Field(default=None, alias="ExtID")
    af_kd_nr: Optional[str] = Field(default=None, alias="AfKdNr")
    error: Optional[WindispoExportError] = Field(default=None, alias="Error")


class WindispoExportCreateJob(WindispoExportBaseModel):
    header: WindispoExportHeader
    job: WindispoExportJob
    jobcount: int = Field(default=1, alias="jobcount")

    @staticmethod
    def _read_text(parent: ET.Element, tag: str) -> Optional[str]:
        element = parent.find(tag)
        if element is None:
            return None
        return element.text

    @classmethod
    def from_xml(cls, xml_text: str) -> "WindispoExportCreateJob":
        root = ET.fromstring(xml_text)
        header = root.find("header")
        job = root.find("job")

        if header is None or job is None:
            raise ValueError("Invalid Windispo export XML: missing header or job")

        payload = {
            "header": {
                "KdNr": cls._read_text(header, "KdNr") or "",
                "MsgTime": cls._read_text(header, "MsgTime") or "",
                "MsgFrom": cls._read_text(header, "MsgFrom") or "",
                "ProcessType": cls._read_text(header, "ProcessType") or "",
                "ProcessedFile": cls._read_text(header, "ProcessedFile") or "",
            },
            "job": {
                "DocID": cls._read_text(job, "DocID"),
                "ExtID": cls._read_text(job, "ExtID"),
                "AfKdNr": cls._read_text(job, "AfKdNr"),
            },
            "jobcount": int(cls._read_text(root, "jobcount") or 1),
        }

        error = job.find("Error")
        if error is not None:
            payload["job"]["Error"] = {
                "ErrorMsg": cls._read_text(error, "ErrorMsg") or ""
            }

        return cls.model_validate(payload)



from pydantic import BaseModel, Field
from typing import Literal, Optional
from datetime import datetime


class Geb(BaseModel):
    satzart: Literal["GEB"] = "GEB"
    referenz: str
    tladenr: str
    aufnr: int
    kunart: int
    menge: Optional[str] = None
    betrag: Optional[str] = None
    waehr: Optional[str] = None
    ucprz: Optional[str] = None
    abreart: Optional[str] = None
    infotext: Optional[str] = None



from pydantic import BaseModel, field_validator
from typing import Optional, Union, Literal
from datetime import datetime


class Posit(BaseModel):
    satzart: Literal["POSIT"] = "POSIT"
    referenz: Optional[str] = None
    tladenr: Optional[str] = None
    aufnr: Optional[int] = None
    aufposnr: Optional[int] = None
    refpos: Optional[str] = None
    tatsgew: Optional[Union[str, float]] = None
    fpflgew: Optional[Union[str, float]] = None
    ean: Optional[str] = None
    artnre: Optional[str] = None
    artnrl: Optional[str] = None
    artikel: Optional[str] = None
    zeichen: Optional[str] = None
    inhalt: Optional[str] = None
    tarifid: Optional[str] = None
    ggutklasse: Optional[str] = None
    ggutunr: Optional[str] = None
    ggutziffer: Optional[str] = None
    ggutgew: Optional[Union[str, float]] = None
    cdmanz: Optional[Union[str, float]] = None
    lmanz: Optional[Union[str, float]] = None
    qmanz: Optional[Union[str, float]] = None
    spanz: Optional[Union[str, float]] = None
    smenge: Optional[Union[str, float]] = None
    imenge: Optional[Union[str, float]] = None
    warenwert: Optional[Union[str, float]] = None
    qualitaet: Optional[int] = None
    wauftragid: Optional[int] = None
    waufposid: Optional[int] = None
    einhgew: Optional[str] = None
    einhvol: Optional[str] = None
    einhlaenge: Optional[str] = None
    einhflaech: Optional[str] = None
    tgew: Optional[Union[str, float]] = None
    hoehe: Optional[Union[str, float]] = None
    breite: Optional[Union[str, float]] = None
    laenge: Optional[Union[str, float]] = None
    taragew: Optional[Union[str, float]] = None
    stapelhoe: Optional[float] = None
    kammer: Optional[str] = None
    inhalt2: Optional[str] = None
    hbuchkto: Optional[str] = None
    kostena: Optional[str] = None
    innenaufnr: Optional[str] = None
    vertrieb: Optional[int] = None
    matart: Optional[str] = None
    matgrp: Optional[str] = None
    sparte: Optional[str] = None
    gklasse: Optional[str] = None
    ggruppe: Optional[str] = None
    verswert: Optional[Union[str, float]] = None
    shilf1: Optional[str] = None
    shilf2: Optional[str] = None
    shilf3: Optional[str] = None
    lhilf1: Optional[int] = None
    lhilf2: Optional[int] = None
    dtmhilf1: Optional[datetime] = None
    dtmhilf2: Optional[datetime] = None
    curhilf1: Optional[Union[str, float]] = None
    curhilf2: Optional[Union[str, float]] = None
    pspelement: Optional[str] = None
    befkat: Optional[str] = None
    fz: Optional[int] = None
    aendstatus: Optional[str] = None
    solttatsgew: Optional[Union[str, float]] = None
    sollpalanze: Optional[Union[str, float]] = None
    sollvpeanz: Optional[Union[str, float]] = None
    sollmeanz: Optional[Union[str, float]] = None
    sollspanz: Optional[Union[str, float]] = None
    beltxt: Optional[str] = None
    leergut: Optional[int] = None
    umweltgef: Optional[str] = None  # Changed from Optional[bool]
    curhilf3: Optional[Union[str, float]] = None
    curhilf4: Optional[Union[str, float]] = None
    curhilf5: Optional[Union[str, float]] = None
    curhilf6: Optional[Union[str, float]] = None
    curhilf7: Optional[Union[str, float]] = None

    @field_validator("dtmhilf1", "dtmhilf2", mode="before")
    @classmethod
    def parse_datetime(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%Y%m%d")
        return value

    @field_validator("aendstatus")
    @classmethod
    def validate_aendstatus(cls, value):
        if value and value not in {"N", "A", "L"}:
            raise ValueError("Invalid Aendstatus value")
        return value

    @field_validator("umweltgef")
    @classmethod
    def validate_umweltgef(cls, value):
        if value in {True, "J", "j"}:
            return "J"
        elif value in {False, "N", "n"}:
            return "N"
        elif value is None:
            return None
        raise ValueError("umweltgef must be 'J', 'N', True, False or None")



from pydantic import BaseModel, Field, validator
from typing import Literal, Optional, Union
from datetime import datetime


class Lademi(BaseModel):
    satzart: Literal["LADEMI"] = "LADEMI"
    referenz: Optional[str] = None
    tladenr: Optional[str] = None
    aufnr: Union[str, int, None] = None
    aufposnr: Union[str, int, None] = None
    palanz: Optional[int] = None
    pal: Optional[str] = None
    laenge: Optional[Union[str, float]] = None
    breite: Optional[Union[str, float]] = None
    hoehe: Optional[Union[str, float]] = None
    eigengew: Optional[Union[str, float]] = None
    nve: Optional[str] = None
    lmart: Union[str, int]
    waufragid: Optional[Union[str, int]] = None
    waufposid: Optional[Union[str, int]] = None
    einhgew: Optional[str] = None
    einhlaenge: Optional[str] = None
    einhbreite: Optional[str] = None
    einhhoehe: Optional[str] = None
    lageplatz: Optional[str] = None
